@@ -170,31 +170,30 @@ class RoutingConf:
         return StaticFile(fpath, content_type)
 
     def get_url_controller(self, path="", method="") -> Tuple[ControllerFunction, Dict, List]:
-        decoded_path = unquote(path)
         # explicitly url matching
-        if decoded_path in self.method_url_mapping[method]:
-            return self.method_url_mapping[method][decoded_path], {}, ()
-        elif decoded_path in self.method_url_mapping["_"]:
-            return self.method_url_mapping["_"][decoded_path], {}, ()
+        if path in self.method_url_mapping[method]:
+            return self.method_url_mapping[method][path], {}, ()
+        elif path in self.method_url_mapping["_"]:
+            return self.method_url_mapping["_"][path], {}, ()
 
         # url with path value matching
-        fun_and_val = self.__try_get_from_path_val(decoded_path, method)
+        fun_and_val = self.__try_get_from_path_val(path, method)
         if fun_and_val is None:
-            fun_and_val = self.__try_get_from_path_val(decoded_path, "_")
+            fun_and_val = self.__try_get_from_path_val(path, "_")
         if fun_and_val is not None:
             return fun_and_val[0], fun_and_val[1], ()
 
         # regexp
-        func_and_groups = self.__try_get_from_regexp(decoded_path, method)
+        func_and_groups = self.__try_get_from_regexp(path, method)
         if func_and_groups is None:
-            func_and_groups = self.__try_get_from_regexp(decoded_path, "_")
+            func_and_groups = self.__try_get_from_regexp(path, "_")
         if func_and_groups is not None:
             return func_and_groups[0], {}, func_and_groups[1]
         # static files
         for k, v in self.res_conf:
-            if decoded_path.startswith(k):
+            if path.startswith(k):
                 def static_fun():
-                    return self._res_(decoded_path, k, v)
+                    return self._res_(path, k, v)
                 return ControllerFunction(func=static_fun), {}, ()
         return None, {}, ()
 
@@ -367,9 +366,9 @@ class AsyncioMixin:
 
     def __init__(self) -> None:
         # All the coroutine tasks will run in this thread.
-        self.__coroutine_thread: threading.Thread = threading.Thread(target=self.coroutine_main, name="coroutine-thread", daemon=True)
-        self.__coroutine_thread.start()
         self.__loop = None
+        self.__coroutine_thread: threading.Thread = threading.Thread(target=self.coroutine_main, name="coroutine-thread", daemon=False)
+        self.__coroutine_thread.start()
         self.__coroutine_tasks = {}
 
     def put_coroutine_task(self, request, task):
@@ -385,7 +384,6 @@ class AsyncioMixin:
         finally:
             loop.run_until_complete(loop.shutdown_asyncgens())
             loop.close()
-            _logger.info("End of the coroutine....")
 
     async def process_request_async(self, request, client_address):
         """Same as in BaseServer but as async.
@@ -408,18 +406,20 @@ class AsyncioMixin:
             self.shutdown_request(request)
 
     def process_request(self, request, client_address):
+        while not self.__loop:
+            # wait for the loop ready
+            time.sleep(0.3)
         asyncio.run_coroutine_threadsafe(self.process_request_async(request, client_address), self.__loop)
 
     def server_close(self):
         super().server_close()
-        self.__loop.stop()
+        self.__loop.call_soon_threadsafe(self.__loop.stop)
         self.__coroutine_thread.join()
 
     def shutdown(self):
         super().shutdown()
-        self.__loop.stop()
+        self.__loop.call_soon_threadsafe(self.__loop.stop)
         self.__coroutine_thread.join()
-        _logger.debug("shutdown....join thread...")
 
 
 class AsyncioMixInHTTPServer(AsyncioMixin, HTTPServer):
@@ -498,12 +498,8 @@ class SimpleDispatcherHttpServer:
             raise
 
     def shutdown(self):
-        def shut():
-            for i in (3, 2, 1, 0):
-                _logger.info(f"server receives a shutdown signal, will shut the server in {i} second(s). ")
-                time.sleep(1)
-            self.server.shutdown()
-        threading.Thread(target=shut, daemon=True).start()
+        # shutdown it in a seperate thread.
+        threading.Thread(target=self.server.shutdown, daemon=True).start()
 
 
 class WSGIProxy(RoutingConf):
